/* Copyright 1998, 2005 The Apache Software Foundation or its licensors, as applicable
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package java.lang;

/**
 * This class must be implemented by the vm vendor. Object is the root of the
 * java class hierarchy. All non-base types respond to the messages defined in
 * this class.
 * 
 */

public class Object {

	/**
	 * Constructs a new instance of this class.
	 * 
	 */
	public Object() {
	}

	/**
	 * Answers a new instance of the same class as the receiver, whose slots
	 * have been filled in with the values in the slots of the receiver.
	 * <p>
	 * Classes which wish to support cloning must specify that they implement
	 * the Cloneable interface, since the implementation checks for this.
	 * 
	 * @return Object a shallow copy of this object.
	 * @exception CloneNotSupportedException
	 *                if the receiver's class does not implement the interface
	 *                Cloneable.
	 */
	protected Object clone() throws CloneNotSupportedException {
        System.out.println("Object.clone() has been called");
        try 
        {
            Class cls = Class.forName("java.lang.Cloneable");
            cls.isInstance(this);
        } 
        catch (Exception e) 
        {
            System.out.println("Object.clone() -- exception = " + e);
            return null;
        }
		return VMObject.clone((Cloneable)this);
	};

	/**
	 * Compares the argument to the receiver, and answers true if they represent
	 * the <em>same</em> object using a class specific comparison. The
	 * implementation in Object answers true only if the argument is the exact
	 * same object as the receiver (==).
	 * 
	 * @param o
	 *            Object the object to compare with this object.
	 * @return boolean <code>true</code> if the object is the same as this
	 *         object <code>false</code> if it is different from this object.
	 * @see #hashCode
	 */
	public boolean equals(Object o) {
        System.out.println("Object.equals() has been called");
		return (o == this);
	}

	/**
	 * Called by the virtual machine when there are no longer any (non-weak)
	 * references to the receiver. Subclasses can use this facility to guarantee
	 * that any associated resources are cleaned up before the receiver is
	 * garbage collected. Uncaught exceptions which are thrown during the
	 * running of the method cause it to terminate immediately, but are
	 * otherwise ignored.
	 * <p>
	 * Note: The virtual machine assumes that the implementation in class Object
	 * is empty.
	 * 
	 * @exception Throwable
	 *                The virtual machine ignores any exceptions which are
	 *                thrown during finalization.
	 */

	protected void finalize() throws Throwable {
        System.out.println("Object.finalize() has been called");
	}

	/**
	 * Answers the unique instance of java.lang.Class which represents the class
	 * of the receiver.
	 * 
	 * @return Class the receiver's Class
	 */
	public final Class getClass() {
        System.out.println("Object.getClass() has been called");
		return VMObject.getClass(this);
	};

	/**
	 * Answers an integer hash code for the receiver. Any two objects which
	 * answer <code>true</code> when passed to <code>.equals</code> must
	 * answer the same value for this method.
	 * 
	 * @return int the receiver's hash.
	 * @see #equals
	 */
	public int hashCode() {
        System.out.println("Object.hashCode() has been called");
		return VMSystem.identityHashCode(this);
	};

	/**
	 * Causes one thread which is <code>wait</code> ing on the receiver to be
	 * made ready to run. This does not guarantee that the thread will
	 * immediately run. The method can only be invoked by a thread which owns
	 * the receiver's monitor.
	 * 
	 * @see #notifyAll
	 * @see #wait()
	 * @see #wait(long)
	 * @see #wait(long,int)
	 * @see java.lang.Thread
	 */
	public final void notify() {
        System.out.println("Object.notify() has been called");
		VMObject.notify(this);
        return;
	};

	/**
	 * Causes all threads which are <code>wait</code> ing on the receiver to
	 * be made ready to run. The threads are scheduled according to their
	 * priorities as specified in class Thread. Between any two threads of the
	 * same priority the one which waited first will be the first thread that
	 * runs after being notified. The method can only be invoked by a thread
	 * which owns the receiver's monitor.
	 * 
	 * @see #notify
	 * @see #wait()
	 * @see #wait(long)
	 * @see #wait(long,int)
	 * @see java.lang.Thread
	 */
	public final void notifyAll() {
        System.out.println("Object.notifyAll() has been called");
		VMObject.notifyAll(this);
        return;
	};

	/**
	 * Answers a string containing a concise, human-readable description of the
	 * receiver.
	 * 
	 * @return String a printable representation for the receiver.
	 */
	public String toString() {
        System.out.println("Object.toString() has been called");
        int hc = VMSystem.identityHashCode(this);
        Integer ihc = new Integer(hc);
		String retVal = new String(ihc.toString());
        return retVal;
	}

	/**
	 * Causes the thread which sent this message to be made not ready to run
	 * pending some change in the receiver (as indicated by <code>notify</code>
	 * or <code>notifyAll</code>). The method can only be invoked by a thread
	 * which owns the receiver's monitor. A waiting thread can be sent
	 * <code>interrupt()</code> to cause it to prematurely stop waiting, so
	 * senders of wait should check that the condition they were waiting for has
	 * been met.
	 * <p>
	 * When the thread waits, it gives up ownership of the receiver's monitor.
	 * When it is notified (or interrupted) it re-acquires the monitor before it
	 * starts running.
	 * 
	 * @exception InterruptedException
	 *                to interrupt the wait.
	 * @see Thread#interrupt
	 * @see #notify
	 * @see #notifyAll
	 * @see #wait(long)
	 * @see #wait(long,int)
	 * @see java.lang.Thread
	 */
	public final void wait() throws InterruptedException {
        System.out.println("Object.wait() has been called");
		VMObject.wait(this, 0, 0);
        return;
	}

	/**
	 * Causes the thread which sent this message to be made not ready to run
	 * either pending some change in the receiver (as indicated by
	 * <code>notify</code> or <code>notifyAll</code>) or the expiration of
	 * the timeout. The method can only be invoked by a thread which owns the
	 * receiver's monitor. A waiting thread can be sent <code>interrupt()</code>
	 * to cause it to prematurely stop waiting, so senders of wait should check
	 * that the condition they were waiting for has been met.
	 * <p>
	 * When the thread waits, it gives up ownership of the receiver's monitor.
	 * When it is notified (or interrupted) it re-acquires the monitor before it
	 * starts running.
	 * 
	 * @param time
	 *            long The maximum time to wait in milliseconds.
	 * @exception InterruptedException
	 *                to interrupt the wait.
	 * @see #notify
	 * @see #notifyAll
	 * @see #wait()
	 * @see #wait(long,int)
	 * @see java.lang.Thread
	 */
	public final void wait(long time) throws InterruptedException {
        System.out.println("Object.wait(time) has been called");
		VMObject.wait(this, time, 0);
        return;
	}

	/**
	 * Causes the thread which sent this message to be made not ready to run
	 * either pending some change in the receiver (as indicated by
	 * <code>notify</code> or <code>notifyAll</code>) or the expiration of
	 * the timeout. The method can only be invoked by a thread which owns the
	 * receiver's monitor. A waiting thread can be sent <code>interrupt()</code>
	 * to cause it to prematurely stop waiting, so senders of wait should check
	 * that the condition they were waiting for has been met.
	 * <p>
	 * When the thread waits, it gives up ownership of the receiver's monitor.
	 * When it is notified (or interrupted) it re-acquires the monitor before it
	 * starts running.
	 * 
	 * @param time
	 *            long The maximum time to wait in milliseconds.
	 * @param frac
	 *            int The fraction of a mSec to wait, specified in nano seconds.
	 * @exception InterruptedException
	 *                to interrupt the wait.
	 * @see #notify
	 * @see #notifyAll
	 * @see #wait()
	 * @see #wait(long)
	 * @see java.lang.Thread
	 */
	public final void wait(long time, int frac) throws InterruptedException {
        System.out.println("Object.wait(long, int) has been called");
		VMObject.wait(this, time, frac);
        return;
	};
}
