/* Copyright 1998, 2005 The Apache Software Foundation or its licensors, as applicable
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package java.lang;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * This class must be implemented by the vm vendor, or the reference
 * implementation can be used if the documented natives are implemented.
 * 
 * This class is the superclass of all classes which can be thrown by the
 * virtual machine. The two direct subclasses represent recoverable exceptions
 * (Exception) and unrecoverable errors (Error). This class provides common
 * methods for accessing a string message which provides extra information about
 * the circumstances in which the throwable was created, and for filling in a
 * walkback (i.e. a record of the call stack at a particular point in time)
 * which can be printed later.
 * 
 * @see Error
 * @see Exception
 * @see RuntimeException
 */
public class Throwable implements java.io.Serializable {
	private static final long serialVersionUID = -3042686055658047285L;

	/**
	 * The message provided when the exception was created.
	 */
	private String detailMessage;

	/**
	 * An object which describes the walkback. This field is stored by the
	 * fillInStackTrace() native, and used by the getStackTraceImpl() native.
	 */
	private transient Object walkback;

	/**
	 * The cause of this Throwable. Null when there is no cause.
	 */
	private Throwable cause = this;

	private StackTraceElement[] stackTrace = null;

	/**
	 * Constructs a new instance of this class with its walkback filled in.
	 */
	public Throwable() {
		super();
        System.out.println("Throwable.<init> -- not fully implemented");
		fillInStackTrace();
	}

	/**
	 * Constructs a new instance of this class with its walkback and message
	 * filled in.
	 * 
	 * @param detailMessage
	 *            String The detail message for the exception.
	 */
	public Throwable(String detailMessage) {
		this();
        System.out.println("Throwable.<init>(String) -- not fully implemented");
		this.detailMessage = detailMessage;
	}

	/**
	 * Constructs a new instance of this class with its walkback, message and
	 * cause filled in.
	 * 
	 * @param detailMessage
	 *            String The detail message for the exception.
	 * @param throwable
	 *            The cause of this Throwable
	 */
	public Throwable(String detailMessage, Throwable throwable) {
		this();
        System.out.println("Throwable.<init>(String, Throwable) -- not fully implemented");
		this.detailMessage = detailMessage;
		cause = throwable;
	}

	/**
	 * Constructs a new instance of this class with its walkback and cause
	 * filled in.
	 * 
	 * @param throwable
	 *            The cause of this Throwable
	 */
	public Throwable(Throwable throwable) {
		this();
        System.out.println("Throwable.<init>(Throwable) -- not fully implemented");
		this.detailMessage = throwable == null ? null : throwable.toString();
		cause = throwable;
	}

	/**
	 * This native must be implemented to use the reference implementation of
	 * this class.
	 * 
	 * Record in the receiver a walkback from the point where this message was
	 * sent. The message is public so that code which catches a throwable and
	 * then <em>re-throws</em> it can adjust the walkback to represent the
	 * location where the exception was re-thrown.
	 * 
	 * @return the receiver
	 */
	//  this is a native instance method, wrong!  ---->public native Throwable fillInStackTrace();
    public Throwable fillInStackTrace() 
    {
        System.out.println("Throwable.fillInStackTrace() -- not fully implemented");
        Throwable thr = VMThrowable.fillInStackTrace(this);
        return thr;
    }

	/**
	 * Answers the extra information message which was provided when the
	 * throwable was created. If no message was provided at creation time, then
	 * answer null.
	 * 
	 * @return String The receiver's message.
	 */
	public String getMessage() {
        System.out.println("Throwable.getMessage() -- not fully implemented");
		return detailMessage;
	}

	/**
	 * Answers the extra information message which was provided when the
	 * throwable was created. If no message was provided at creation time, then
	 * answer null. Subclasses may override this method to answer localized text
	 * for the message.
	 * 
	 * @return String The receiver's message.
	 */
	public String getLocalizedMessage() {
        System.out.println("Throwable.getLocalizedMessage() -- not fully implemented");
		return getMessage();
	}

	/**
	 * This native must be implemented to use the reference implementation of
	 * this class. The result of this native is cloned, and returned from the
	 * public API getStackTrace().
	 * 
	 * Answers an array of StackTraceElement. Each StackTraceElement represents
	 * a entry on the stack.
	 * 
	 * @return an array of StackTraceElement representing the stack
	 */
	//  this is a native instance method, wrong!  ----> private native StackTraceElement[] getStackTraceImpl();
    private StackTraceElement[] getStackTraceImpl()
    {
        System.out.println("Throwable.getStackTraceImpl() -- not implemented");
        // VMThrowable.getStackTrace(this);
        return null;
    }
	/**
	 * Answers an array of StackTraceElement. Each StackTraceElement represents
	 * a entry on the stack.
	 * 
	 * @return an array of StackTraceElement representing the stack
	 */
	public StackTraceElement[] getStackTrace() {
        System.out.println("Throwable.getStackTrace() -- not fully implemented");
		return (StackTraceElement[]) getInternalStackTrace().clone();
	}

	/**
	 * Sets the array of StackTraceElements. Each StackTraceElement represents a
	 * entry on the stack. A copy of this array will be returned by
	 * getStackTrace() and printed by printStackTrace().
	 * 
	 * @param trace
	 *            The array of StackTraceElement
	 */
	public void setStackTrace(StackTraceElement[] trace) {
        System.out.println("Throwable.setStackTrace() -- not fully implemented");
		StackTraceElement[] newTrace = (StackTraceElement[]) trace.clone();
		for (int i = 0; i < newTrace.length; i++)
			if (newTrace[i] == null)
				throw new NullPointerException();
		stackTrace = newTrace;
	}

	/**
	 * Outputs a printable representation of the receiver's walkback on the
	 * System.err stream.
	 */
	public void printStackTrace() {
        System.out.println("Throwable.printStackTrace() -- not fully implemented");
		printStackTrace(System.err);
	}

	/**
	 * Count the number of duplicate stack frames, starting from
	 * the end of the stack.
	 * 
	 * @param currentStack a stack to compare
	 * @param parentStack a stack to compare
	 *  
	 * @return the number of duplicate stack frames.
	 */
	private static int countDuplicates(StackTraceElement[] currentStack,
			StackTraceElement[] parentStack) {
        System.out.println("Throwable.countDuplicates() -- was called");
		int duplicates = 0;
		int parentIndex = parentStack.length;
		for (int i = currentStack.length; --i >= 0 && --parentIndex >= 0;) {
			StackTraceElement parentFrame = parentStack[parentIndex];
			if (parentFrame.equals(currentStack[i])) {
				duplicates++;
			} else {
				break;
			}
		}
		return duplicates;
	}

	/**
	 * Answers an array of StackTraceElement. Each StackTraceElement represents
	 * a entry on the stack. Cache the stack trace in the stackTrace field, returning
	 * the cached field when it has already been initialized.
	 *
	 * @return an array of StackTraceElement representing the stack
	 */
	private StackTraceElement[] getInternalStackTrace() {
        System.out.println("Throwable.getInternalStackTrace() -- not fully implemented");
		if (stackTrace == null)
			stackTrace = getStackTraceImpl();
		return stackTrace;
	}

	/**
	 * Outputs a printable representation of the receiver's walkback on the
	 * stream specified by the argument.
	 * 
	 * @param err
	 *            PrintStream The stream to write the walkback on.
	 */
	public void printStackTrace(PrintStream err) {
		err.println(toString());
		// Don't use getStackTrace() as it calls clone()
		// Get stackTrace, in case stackTrace is reassigned
		StackTraceElement[] stack = getInternalStackTrace();
		for (int i = 0; i < stack.length; i++)
			err.println("\tat " + stack[i]);

		StackTraceElement[] parentStack = stack;
		Throwable throwable = getCause();
		while (throwable != null) {
			err.print("Caused by: ");
			err.println(throwable);
			StackTraceElement[] currentStack = throwable
					.getInternalStackTrace();
			int duplicates = countDuplicates(currentStack, parentStack);
			for (int i = 0; i < currentStack.length - duplicates; i++)
				err.println("\tat " + currentStack[i]);
			if (duplicates > 0) {
				err.println("\t... " + duplicates + " more");
			}
			parentStack = currentStack;
			throwable = throwable.getCause();
		}
	}

	/**
	 * Outputs a printable representation of the receiver's walkback on the
	 * writer specified by the argument.
	 * 
	 * @param err
	 *            PrintWriter The writer to write the walkback on.
	 */
	public void printStackTrace(PrintWriter err) {
		err.println(toString());
		// Don't use getStackTrace() as it calls clone()
		// Get stackTrace, in case stackTrace is reassigned
		StackTraceElement[] stack = getInternalStackTrace();
		for (int i = 0; i < stack.length; i++)
			err.println("\tat " + stack[i]);

		StackTraceElement[] parentStack = stack;
		Throwable throwable = getCause();
		while (throwable != null) {
			err.print("Caused by: ");
			err.println(throwable);
			StackTraceElement[] currentStack = throwable
					.getInternalStackTrace();
			int duplicates = countDuplicates(currentStack, parentStack);
			for (int i = 0; i < currentStack.length - duplicates; i++)
				err.println("\tat " + currentStack[i]);
			if (duplicates > 0) {
				err.println("\t... " + duplicates + " more");
			}
			parentStack = currentStack;
			throwable = throwable.getCause();
		}
	}

	/**
	 * Answers a string containing a concise, human-readable description of the
	 * receiver.
	 * 
	 * @return String a printable representation for the receiver.
	 */
	public String toString() {
		String msg = getLocalizedMessage();
		String name = getClass().getName();
		if (msg == null) {
			return name;
		}
		return new StringBuffer(name.length() + 2 + msg.length()).append(name)
				.append(": ").append(msg).toString();
	}

	/**
	 * Initialize the cause of the receiver. The cause cannot be reassigned.
	 * 
	 * @param throwable
	 *            The cause of this Throwable
	 * 
	 * @exception IllegalArgumentException
	 *                when the cause is the receiver
	 * @exception IllegalStateException
	 *                when the cause has already been initialized
	 * 
	 * @return the receiver.
	 */
	public synchronized Throwable initCause(Throwable throwable) {
		if (cause == this) {
			if (throwable != this) {
				cause = throwable;
				return this;
			}
			throw new IllegalArgumentException("Cause cannot be the receiver");
		}
		throw new IllegalStateException("Cause already initialized");
	}

	/**
	 * Answers the cause of this Throwable, or null if there is no cause.
	 * 
	 * @return Throwable The receiver's cause.
	 */
	public Throwable getCause() {
		if (cause == this)
			return null;
		return cause;
	}

	private void writeObject(ObjectOutputStream s) throws IOException {
		// ensure the stackTrace field is initialized
		getInternalStackTrace();
		s.defaultWriteObject();
	}
}
